#include "vector.h"
#include <stdio.h>

Vector *new(int nBits)
{
    Vector *pVector = (Vector *) malloc(sizeof (Vector));
    pVector->nBits = nBits;

    pVector->nInts = (nBits + sizeof(int) - 1) / sizeof(int);
    pVector->pInts = (int *) calloc(pVector->nInts, sizeof(int));
    
    return pVector;
}

void DESTROY(Vector *pVector)
{
    free(pVector->pInts);
    free(pVector);
}

void dump(Vector *pVector)
{
    int i;

    for (i=0; i<pVector->nInts; i++)
	fprintf(stderr, "%2d %08x\n", i, pVector->pInts[i]);
}

int top(Vector *pVector)
{
    return pVector->nBits - 1;
}

void insert(Vector *pVector, int n)
{
    int q, r;
    
    q = n / sizeof (int);
    r = n % sizeof (int);

    pVector->pInts[q] |= 1<<r;
}

void Remove(Vector *pVector, int n)
{
    int q, r;
    
    q = n / sizeof (int);
    r = n % sizeof (int);

    pVector->pInts[q] &= ~(1<<r);
}

int member(Vector *pVector, int n)
{
    int q, r;
    
    q = n / sizeof (int);
    r = n % sizeof (int);

    return pVector->pInts[q] & (1<<r);
}

Vector *Union(Vector *pA, Vector *pB)
{
    Vector *pC;
    int i, nA, nB;

    nA = pA->nInts;
    nB = pB->nInts;

    if (nA < nB)
	return Union(pB, pA);

    pC = new(pA->nBits);

    for (i=0; i<nB; i++)
	pC->pInts[i] = pA->pInts[i] | pB->pInts[i];

    for (   ; i<nA; i++)
	pC->pInts[i] = pA->pInts[i];

    return pC;
}

Vector *intersect(Vector *pA, Vector *pB)
{
    Vector *pC;
    int i, nA, nB;
   
    nA = pA->nInts;
    nB = pB->nInts;

    if (nA > nB)
	return intersect(pB, pA);

    pC = new(pA->nBits);

    for (i=0; i<nA; i++)
	pC->pInts[i] = pA->pInts[i] & pB->pInts[i];

    return pC;
}


